#!/usr/bin/perl -w

##
# File:         comp2bundles.pl
# Description:  filter to generate html/bundles file from RedHat comps
# Author:       Bryan Gartner   < bryan.gartner@hp.com >
#		inspired by Paul Telford's readcomps.pl
##

# © Copyright 2000-2007 Hewlett-Packard Development Company, L.P

# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.

# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
# or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
# for more details.

# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 675 Mass Ave, Cambridge, MA 02139, USA.

# This is a simple filter script:

# it reads older RedHat-based distribution comps (non-xml) content
# as standard input and generates:
#   - simple HTML fragments on standard output
#   - a bundle list on standard error


# typical usage is to

# ./comps2bundles.pl < RedHat/base/comp > @prefix@/html/bundles/<distro>-<version>.html

print "<DL>\n";

while (<>) {

    # remove leading/trailing whitespace, and hidden attributes
    chomp;
    s/^\s+//;
    s/\s+$//;
    s/ --hide//g;
	
    # skip header
    next if( /^4$/ );

    # find bundles
    if( /^[01] (.*?) {/ ) {
        print STDERR "$1\n";
        print " <br>\n <DT><A NAME=\"$1\"><b>$1</b></A></DT>\n";
    }
    next if( /^[01] (.*?) \{/ );

    # find nested bundles
    if( /^\? (.*?) {/ ) {
        print "  <DD><A HREF=\"\#$1\">$1</A></DD>\n";
    }
    next if( /^\? (.*?) \{/ );

    # skip bundle closures
    next if( /^}/ );

    # conditional packages
    if( /:/ ) {
        next if( /}/ );
        ($undef, $foo) = split(/: /, $_);
        if( ( $foo =~ /[01] (.*?) {/ ) || ( $foo =~ /\? (.*?) {/ ) ) {
            print "  <DD><A HREF=\"\#$1\">$1</A></DD>\n";
        } else {
            print "  <DD>$foo [ <font size=\"-1\"><i>$undef</i></font> ]</DD>\n";
        }
    } else {
        if( /\@ / ) {
            s/\@ //g;
            print "  <DD><A HREF=\"\#$_\">$_ </A></DD>\n";
        } else {
            print "  <DD>$_ </DD>\n";
        }
    }
}

print "</DL>\n";
